#include "main.h"

#ifndef __PMap__
#define __PMap__

struct PMap{
  PMap() : 
    vectorField("var:x,y,z;fun:y,z,1-y-0.5*x^2;"),
    solver(vectorField,15),
    section(3,2),
    pm(solver,section),
    tm(solver)
  {}
 
  IVector operator()(IVector u, int iterations){
    capd3::C0HOTripletonSet s(embed(u));
    return project( pm(s,iterations) );
  }
  
  IVector operator()(capd3::IVector x, capd3::IMatrix C, capd3::IVector r, capd3::IMatrix Q, capd3::IVector y, int iterations){
    capd3::C0TripletonSet s(x,C,r);
    interval returnTime;
    return project( pm(s,y,Q,returnTime,iterations) );
  }

  capd3::IVector operator()(capd3::IVector x, capd3::IMatrix C, capd3::IVector r, capd3::IMatrix Q, capd3::IVector y, interval T){
    capd3::C0TripletonSet s(x,C,r);
    tm(T,s);
    return s.affineTransformation(Q,y);
  }

  IVector operator()(capd3::IVector x, capd3::IMatrix C, capd3::IVector r, int iterations){
    capd3::C0TripletonSet s(x,C,r);
    return project( pm(s,iterations) );
  }

  IMatrix derivative(capd3::IVector x, capd3::IMatrix C, capd3::IVector r, int iterations){
    capd3::C1Rect2Set s(capd3::C1Rect2Set::C0BaseSet(x,C,r),capd3::C1Rect2Set::C1BaseSet(C));
    capd3::IMatrix DQ(3,3);
    capd3::IVector y = pm(s,DQ,iterations);
    DQ = pm.computeDP(y,DQ);

    IMatrix DP(2,2);
    DP[0][0] = DQ[0][0];
    DP[0][1] = DQ[0][1];
    DP[1][0] = DQ[1][0];
    DP[1][1] = DQ[1][1];
    return DP;
  }

  capd3::IVector embed(IVector u){
    return capd3::IVector ({u[0],u[1],interval(0.)}); 
  }

  IVector project(capd3::IVector u){
    return IVector({u[0],u[1]});
  }

  capd3::IMatrix embed(IVector u, IVector s){
    capd3::IMatrix Q(3,3);
    Q[0][0] = u[0]; Q[1][0] = u[1];
    Q[0][1] = s[0]; Q[1][1] = s[1];
    Q[2][2] = 1.;
    return Q;
  }

  capd3::IMap vectorField;
  capd3::IOdeSolver solver;
  capd3::ICoordinateSection section;
  capd3::IPoincareMap pm;
  capd3::ITimeMap tm;
};

#endif
