#include "main.h"

//###########################################################//
/**
 * This function initializes all h-sets used in the computations. 
 * The coordinates of sets are given in both papers JDE'2003 and SIADS'2005.
 */ 
void initSets()
{
  // Five h-sets on which symbolic dynamics is built (JDE'2003). 
	IVector center, u,s;

	interval S(100.);
	center[0] = interval(0.); center[1] = interval(155.)/S;
	u[0] = interval(14.)/S; u[1] = interval(6.)/S;
	s[0] = interval(-14.)/S; s[1] = interval(6.)/S;
	N[0] = TripleSet(center,s,u);

	center[0] = interval(0.); center[1]=interval(51.)/S;
	u[0] = interval(9.)/S; u[1] = interval(13.)/S;
	s[0] = interval(-9.)/S; s[1] = interval(13.)/S;
	N[1] = TripleSet(center,s,u);

	center[0] = interval(141.)/S; center[1] = interval(97.)/S;
	u[0] = interval(6.)/S; u[1] = interval(5.)/S;
	s[0] = interval(-6.)/S; s[1] = interval(5.)/S;
	N[2] = TripleSet(center,s,u);

	center[0] = interval(0.); center[1] = interval(-235.)/S;
	u[0] = interval(6.)/S;  u[1] = interval(10.)/S;
	s[0] = interval(-6.)/S; s[1] = interval(10.)/S;
	N[3] = TripleSet(center,s,u);

  // This is set M from SIADS'2005.
  center = -interval(84)/interval(100) * u + center;
	u/=10;
	M = TripleSet(center,s,u);

	// Back to JDE'2003.
  center[0] = interval(-141.)/S; center[1] = interval(97.)/S;
	s[0] = interval(-6.)/S; s[1] = interval(5.)/S;
	u[0] = interval(6.)/S; u[1] = interval(5.)/S;
	N[4] = TripleSet(center,s,u);
   
  // The remaining sets are those from SIADS'2005.
  // Small h-sets centered at periodic points S1 and S2.
  // We check cone conditions on them
  
  // Hyperbolic set H1
	IVector fuzzy;
	center[0] = interval(0.); center[1] = interval(1.5259617305037);
	fuzzy = center;
	fuzzy[1] += interval(-4e-13,4e-13);
	s[0] = -1.;
	s[1] = 0.4574907884495629796;
	u = s;
	u[0] = -u[0];
	u *= 0.0003;
	s *= 0.0003;
	H1 = TripleSet(fuzzy,s,u);

  // Larger set G1
	u *= 25;
	s *= 25;
	G1 = TripleSet(center,s,u);

  // Hyperbolic set H2
	center[0] = interval(0.); center[1] = interval(0.5000256485352);
	fuzzy = center;
	fuzzy[1] += interval(-4e-13,4e-13);
	s[0] = -1.;
	s[1] = 0.453892728538218937;
	u = s;
	u[0] = -u[0];
	u *= 0.00003;
	s *= 0.00003;
	H2 = TripleSet(fuzzy,s,u);

  // Larger set G2
	u *= 90;
	s *= 90;
	G2 = TripleSet(center,s,u);

  // 3-dimensional h-set centered at equilibrium.
  capd3::IVector Ic(-sqrt(interval(2.)),0.0,0.0);
   
  capd3::IVector Iu=capd3::IVector(
             interval(1e+16),
             interval(8340388297674541.),
             interval(6956207695598643.)
          )/interval(1e+16);

  capd3::IVector Is1=capd3::IVector(
             0.,
             interval(12335783628006363.),
             interval(-10288522541366950.)
          )/interval(1e+16);

  capd3::IVector Is2=capd3::IVector(
             interval(1e+16),
             interval(-4170194148837272.),
             interval(-13478103847799320.)
          )/interval(1e+16);
          
  interval size = interval(33)/interval(100);
  
  H = HSet3D(Ic,Iu*size,Is1*size,Is2*size);
}

